@@ -2,7 +2,7 @@
 #
 # Copyright 2014 Knowledge Economy Developments Ltd
 # Copyright 2014 David Wells
-# 
+#
 # Henry Gomersall
 # heng@kedevelopments.co.uk
 # David Wells
@@ -41,7 +41,7 @@
 Overview
 """"""""
 
-This module contains a set of functions that return 
+This module contains a set of functions that return
 :class:`pyfftw.FFTW` objects.
 
 The interface to create these objects is mostly the same as
@@ -91,13 +91,20 @@ complexity. This results in a copy being made.
 Although the array that is internal to the :class:`pyfftw.FFTW` object
 will be correctly loaded with the values within the input array, it is
 not necessarily the case that the internal array *is* the input array.
-The actual internal input array can always be retrieved with 
+The actual internal input array can always be retrieved with
 :attr:`pyfftw.FFTW.input_array`.
 
+The behaviour of the ``norm`` option in all builder routines matches that of
+the corresponding numpy functions.  In short, if ``norm`` is ``None``, then the
+output from the forward DFT is unscaled and the inverse DFT is scaled by 1/N,
+where N is the product of the lengths of input array on which the FFT is taken.
+If ``norm == 'ortho'``, then the output of both forward and inverse DFT
+operations are scaled by 1/sqrt(N).
+
 **Example:**
 
 .. doctest::
-    
+
     >>> import pyfftw
     >>> a = pyfftw.empty_aligned(4, dtype='complex128')
     >>> fft = pyfftw.builders.fft(a)
@@ -105,7 +112,7 @@ The actual internal input array can always be retrieved with
     >>> fft() # returns the output
     array([ 10.+0.j,  -2.+2.j,  -2.+0.j,  -2.-2.j])
 
-More examples can be found in the :doc:`tutorial </sphinx/tutorial>`.
+More examples can be found in the :doc:`tutorial </source/tutorial>`.
 
 Supported Functions and Caveats
 """""""""""""""""""""""""""""""
@@ -139,7 +146,7 @@ to note from this is that the precision of the transform matches the
 precision of the input array. So, if a single precision input array is
 passed in, then a single precision transform will be used.
 
-The second caveat is that repeated axes are handled differently; with 
+The second caveat is that repeated axes are handled differently; with
 the returned :class:`pyfftw.FFTW` object, axes that are repeated in the
 axes argument are considered only once, as compared to :mod:`numpy.fft`
 in which repeated axes results in the DFT being taken along that axes
@@ -170,14 +177,14 @@ following additional keyword arguments:
   offered for the multi-dimensional inverse real transforms, as FFTW is
   unable to not overwrite the input in that case.
 
-* ``planner_effort``: A string dictating how much effort is spent 
+* ``planner_effort``: A string dictating how much effort is spent
   in planning the FFTW routines. This is passed to the creation
-  of the :class:`pyfftw.FFTW` object as an entry in the flags list. 
+  of the :class:`pyfftw.FFTW` object as an entry in the flags list.
   They correspond to flags passed to the :class:`pyfftw.FFTW` object.
 
-  The valid strings, in order of their increasing impact on the time 
+  The valid strings, in order of their increasing impact on the time
   to compute  are:
-  ``'FFTW_ESTIMATE'``, ``'FFTW_MEASURE'`` (default), ``'FFTW_PATIENT'``
+  ``'FFTW_ESTIMATE'``, ``config.PLANNER_EFFORT`` (default), ``'FFTW_PATIENT'``
   and ``'FFTW_EXHAUSTIVE'``.
 
   The `Wisdom
@@ -219,17 +226,17 @@ following additional keyword arguments:
   memory before performing the transform on it. If the array is not
   contiguous, it is copied into an interim array. This is because it
   is often faster to copy the data before the transform and then transform
-  a contiguous array than it is to try to take the transform of a 
+  a contiguous array than it is to try to take the transform of a
   non-contiguous array. This is particularly true in conjunction with
-  the ``auto_align_input`` argument which is used to make sure that the 
+  the ``auto_align_input`` argument which is used to make sure that the
   transform is taken of an aligned array.
 
-  Like ``auto_align_input``, If a new array is created, it is 
-  up to the calling code to acquire that new input array using 
+  Like ``auto_align_input``, If a new array is created, it is
+  up to the calling code to acquire that new input array using
   :attr:`pyfftw.FFTW.input_array`.
 
-* ``avoid_copy``: By default, these functions will always create a copy 
-  (and sometimes more than one) of the passed in input array. This is 
+* ``avoid_copy``: By default, these functions will always create a copy
+  (and sometimes more than one) of the passed in input array. This is
   because the creation of the :class:`pyfftw.FFTW` object generally
   destroys the contents of the input array. Setting this argument to
   ``True`` will try not to create a copy of the input array, likely
@@ -245,7 +252,7 @@ following additional keyword arguments:
 
   * The dtypes are incompatible with the FFT routine.
 
-  * The ``auto_contiguous`` or ``auto_align`` flags are True and 
+  * The ``auto_contiguous`` or ``auto_align`` flags are True and
     the input array is not already contiguous or aligned.
 
   This argument is distinct from ``overwrite_input`` in that it only
@@ -256,164 +263,179 @@ The exceptions raised by each of these functions are as per their
 equivalents in :mod:`numpy.fft`, or as documented above.
 '''
 
-from ._utils import _precook_1d_args, _Xfftn
+from ._utils import (_precook_1d_args, _Xfftn, _norm_args, _default_effort,
+                     _default_threads)
 
 __all__ = ['fft','ifft', 'fft2', 'ifft2', 'fftn',
-           'ifftn', 'rfft', 'irfft', 'rfft2', 'irfft2', 'rfftn', 
+           'ifftn', 'rfft', 'irfft', 'rfft2', 'irfft2', 'rfftn',
            'irfftn']
 
 
-def fft(a, n=None, axis=-1, overwrite_input=False, 
-        planner_effort='FFTW_MEASURE', threads=1,
+def fft(a, n=None, axis=-1, overwrite_input=False,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
+        avoid_copy=False, norm=None):
     '''Return a :class:`pyfftw.FFTW` object representing a 1D FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.fft`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.fft`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
     inverse = False
     real = False
 
     s, axes = _precook_1d_args(a, n, axis)
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 def ifft(a, n=None, axis=-1, overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing a 1D 
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing a 1D
     inverse FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.ifft`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.ifft`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
     inverse = True
     real = False
-    
+
     s, axes = _precook_1d_args(a, n, axis)
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 
 def fft2(a, s=None, axes=(-2,-1), overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
+        avoid_copy=False, norm=None):
     '''Return a :class:`pyfftw.FFTW` object representing a 2D FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.fft2`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.fft2`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
-    
+
     inverse = False
     real = False
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 def ifft2(a, s=None, axes=(-2,-1), overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing a 
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing a
     2D inverse FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.ifft2`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.ifft2`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
-    
+
     inverse = True
     real = False
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 
 def fftn(a, s=None, axes=None, overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
+        avoid_copy=False, norm=None):
     '''Return a :class:`pyfftw.FFTW` object representing a n-D FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.fftn`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.fftn`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
 
     inverse = False
     real = False
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 def ifftn(a, s=None, axes=None, overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing an n-D 
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing an n-D
     inverse FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.ifftn`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.ifftn`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
 
     inverse = True
     real = False
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 def rfft(a, n=None, axis=-1, overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing a 1D 
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing a 1D
     real FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.rfft`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.rfft`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
 
     inverse = False
     real = True
-    
+
     s, axes = _precook_1d_args(a, n, axis)
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 def irfft(a, n=None, axis=-1, overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing a 1D 
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing a 1D
     real inverse FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.irfft`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.irfft`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
@@ -422,83 +444,43 @@ def irfft(a, n=None, axis=-1, overwrite_input=False,
     real = True
 
     s, axes = _precook_1d_args(a, n, axis)
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 def rfft2(a, s=None, axes=(-2,-1), overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing a 2D 
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing a 2D
     real FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.rfft2`; 
-    the rest of the arguments are documented 
+
+    The first three arguments are as per :func:`numpy.fft.rfft2`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
     inverse = False
     real = True
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 def irfft2(a, s=None, axes=(-2,-1),
-        planner_effort='FFTW_MEASURE', threads=1,
+        planner_effort=None, threads=None,
         auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing a 2D 
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing a 2D
     real inverse FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.irfft2`; 
-    the rest of the arguments are documented 
-    :ref:`in the module docs <builders_args>`.
-    '''
 
-
-    inverse = True
-    real = True
-
-    overwrite_input = True    
-
-    return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
-
-
-def rfftn(a, s=None, axes=None, overwrite_input=False,
-        planner_effort='FFTW_MEASURE', threads=1,
-        auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing an n-D 
-    real FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.rfftn`; 
-    the rest of the arguments are documented 
-    :ref:`in the module docs <builders_args>`.
-    '''
-
-
-    inverse = False
-    real = True
-
-    return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
-
-
-def irfftn(a, s=None, axes=None,
-        planner_effort='FFTW_MEASURE', threads=1,
-        auto_align_input=True, auto_contiguous=True,
-        avoid_copy=False):
-    '''Return a :class:`pyfftw.FFTW` object representing an n-D 
-    real inverse FFT.
-    
-    The first three arguments are as per :func:`numpy.fft.rfftn`; 
-    the rest of the arguments are documented 
+    The first three arguments are as per :func:`numpy.fft.irfft2`;
+    the rest of the arguments are documented
     :ref:`in the module docs <builders_args>`.
     '''
 
@@ -507,10 +489,57 @@ def irfftn(a, s=None, axes=None,
     real = True
 
     overwrite_input = True
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
 
     return _Xfftn(a, s, axes, overwrite_input, planner_effort,
-            threads, auto_align_input, auto_contiguous, 
-            avoid_copy, inverse, real)
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
 
 
+def rfftn(a, s=None, axes=None, overwrite_input=False,
+        planner_effort=None, threads=None,
+        auto_align_input=True, auto_contiguous=True,
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing an n-D
+    real FFT.
 
+    The first three arguments are as per :func:`numpy.fft.rfftn`;
+    the rest of the arguments are documented
+    :ref:`in the module docs <builders_args>`.
+    '''
+
+
+    inverse = False
+    real = True
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
+
+    return _Xfftn(a, s, axes, overwrite_input, planner_effort,
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
+
+
+def irfftn(a, s=None, axes=None,
+        planner_effort=None, threads=None,
+        auto_align_input=True, auto_contiguous=True,
+        avoid_copy=False, norm=None):
+    '''Return a :class:`pyfftw.FFTW` object representing an n-D
+    real inverse FFT.
+
+    The first three arguments are as per :func:`numpy.fft.rfftn`;
+    the rest of the arguments are documented
+    :ref:`in the module docs <builders_args>`.
+    '''
+
+
+    inverse = True
+    real = True
+
+    overwrite_input = True
+    planner_effort = _default_effort(planner_effort)
+    threads = _default_threads(threads)
+
+    return _Xfftn(a, s, axes, overwrite_input, planner_effort,
+            threads, auto_align_input, auto_contiguous,
+            avoid_copy, inverse, real, **_norm_args(norm))
